/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * Copyright (C) 2007 Ruslan P. Sorokin (sorokin@oogis.ru)
 * and Object-Oriented Geo-Information System Research Laboratory (http://www.oogis.ru)
 * All Rights Reserved.
 */
package ru.oogis.dro;

import java.io.FileWriter;
import java.io.IOException;
import java.io.PipedReader;
import java.io.PipedWriter;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import org.drools.compiler.PackageBuilder;

import edu.stanford.smi.protege.model.Instance;

public class DrlEmitter implements Runnable {

	Instance packInst;
	PipedWriter pipw;
	PipedReader pipr;
	String dsl;
	PipedWriter pipw_dsl;
	PipedReader pipr_dsl;
	
	public DrlEmitter(Instance packInst) {
		this.packInst = packInst;
		Instance dslins = (Instance) packInst.getOwnSlotValue(KB.s_dsl);
		if(dslins != null)
			dsl = (String) dslins.getOwnSlotValue(KB.s_source);
	}
	
	@SuppressWarnings("unchecked")
	public void emitPrologue(Writer wr) throws IOException{
		Object pkg = packInst.getOwnSlotValue(KB.s_title);
		wr.write(DrlParser.PACKAGE+" "+pkg+"\n\n");
		Object exp = packInst.getOwnSlotValue(KB.s_expander);
		if(exp != null)
			wr.write(DrlParser.EXPANDER+" "+exp+"\n\n");
		Collection imps = packInst.getOwnSlotValues(KB.s_imports);
		for (Iterator iter = imps.iterator(); iter.hasNext();) {
			Object imp = iter.next();
			wr.write(DrlParser.IMPORT+" "+imp+"\n");
		}
		Collection glbs = packInst.getOwnSlotValues(KB.s_globals);
		if(!glbs.isEmpty())
			wr.write("\n");
		for (Iterator iter = glbs.iterator(); iter.hasNext();) {
			Instance glb = (Instance) iter.next();
			Object title = glb.getOwnSlotValue(KB.s_title);
			Object clazz = glb.getOwnSlotValue(KB.s_class);
			if(title != null && clazz != null)
				wr.write(DrlParser.GLOBAL+" "+clazz+" "+title+"\n");
		}
		Collection funs = packInst.getOwnSlotValues(KB.s_functions);
		if(!funs.isEmpty())
			wr.write("\n");
		for (Iterator iter = funs.iterator(); iter.hasNext();) {
			Instance fun = (Instance) iter.next();
			String title = (String) fun.getOwnSlotValue(KB.s_title);
			String parms = (String) fun.getOwnSlotValue(KB.s_params);
			String sourc = (String) fun.getOwnSlotValue(KB.s_source);
			String retyp = (String) fun.getOwnSlotValue(KB.s_return_type);
			if(title != null && parms != null && sourc != null && retyp != null){
				if(!parms.startsWith("("))
					parms = "( "+parms+" )";
				if(!sourc.startsWith("{"))
					sourc = "{ "+sourc+" }";
				wr.write(DrlParser.FUNCTION+" "+retyp+" "+title+parms+"\n"+sourc+"\n\n");
			}
		}
	}
	
	@SuppressWarnings("unchecked")
	public void emitRules(Writer wr) throws IOException{
		emitPrologue(wr);
		Collection rls = packInst.getOwnSlotValues(KB.s_rules);
		for (Iterator iter = rls.iterator(); iter.hasNext();) {
			Instance rule = (Instance) iter.next();
			emitRule(rule, wr);
		}
		wr.close();
	}
	
	public void emitRules(String path) throws IOException{
		emitRules(new FileWriter(path));
	}
	
	public void emitRule(Instance rule, Writer wr) throws IOException{
		Object rnm = rule.getOwnSlotValue(KB.s_title);
		wr.write(DrlParser.RULE+" \""+rnm+"\"\n\t");
		Object sln = rule.getOwnSlotValue(KB.s_salience);
		if(sln != null)
			wr.write(DrlParser.SALIENCE+" "+sln+"\n\t");
		Object nlp = rule.getOwnSlotValue(KB.s_no_loop);
		if(nlp != null)
			wr.write(DrlParser.NO_LOOP+" "+nlp+"\n\t");
		Object afs = rule.getOwnSlotValue(KB.s_auto_focus);
		if(afs != null)
			wr.write(DrlParser.AUTO_FOCUS+" "+afs+"\n\t");
		Object agp = rule.getOwnSlotValue(KB.s_agenda_group);
		if(agp != null)
			wr.write(DrlParser.AGENDA_GROUP+" \""+agp+"\"\n\t");
		Object acgp = rule.getOwnSlotValue(KB.s_activation_group);
		if(acgp != null)
			wr.write(DrlParser.ACTIVATION_GROUP+" \""+acgp+"\"\n\t");
		Object def = rule.getOwnSlotValue(KB.s_date_effective);
		if(def != null)
			wr.write(DrlParser.DATE_EFFECTIVE+" \""+def+"\"\n\t");
		Object dex = rule.getOwnSlotValue(KB.s_date_expires);
		if(dex != null)
			wr.write(DrlParser.DATE_EXPIRES+" \""+dex+"\"\n\t");
		Object dia = rule.getOwnSlotValue(KB.s_dialect);
		if(dia != null)
			wr.write(DrlParser.DIALECT+" \""+dia+"\"\n\t");
		Object drn = rule.getOwnSlotValue(KB.s_duration);
		if(drn != null)
			wr.write(DrlParser.DURATION+" "+drn+"\n\t");
		Object whn = rule.getOwnSlotValue(KB.s_when);
		if(whn != null)
			wr.write(DrlParser.WHEN+"\n"+whn+"\n\t");
		Object thn = rule.getOwnSlotValue(KB.s_then);
		if(thn != null)
			wr.write(DrlParser.THEN+"\n"+thn+"\n");
		wr.write(DrlParser.END+"\n\n");
	}
	
	public void addPackageToBuilder(PackageBuilder builder) throws Exception{
		pipw = new PipedWriter();
		pipr = new PipedReader(pipw);
		if(dsl == null){
			new Thread(this).start();
			builder.addPackageFromDrl(pipr);
		} else { // start pumping dsl
			pipw_dsl = new PipedWriter();
			pipr_dsl = new PipedReader(pipw_dsl);
			Thread dsltd = new Thread() {
				public void run(){
					try {
						pipw_dsl.write(dsl);
						pipw_dsl.close();
					} catch (IOException e) {
						e.printStackTrace();
					}
				}
			};
			dsltd.start();
			new Thread(this).start();
			builder.addPackageFromDrl(pipr, pipr_dsl);
		}
	}
	
	public void run(){
		try {
			emitRules(pipw);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	@SuppressWarnings("unchecked")
	public static void main(String[] args) throws Exception {
		if(args.length < 3){
			System.out.println("Usage: java -cp .. ru.oogis.dro.DrlEmitter <path-to-project> <package> <path-to-drl-file>");
		}
		Collection errors = new ArrayList();
		System.out.println("load errors "+errors.size());
		if(errors.size() > 0){
			for (Iterator iter = errors.iterator(); iter.hasNext();) {
				Object element = iter.next();
				System.out.println("error: "+element);
			}
			return;
		}
		Collection pins = KB.c_Package.getInstances();
		for (Iterator iter = pins.iterator(); iter.hasNext();) {
			Instance pk = (Instance) iter.next();
			Object pnam = pk.getOwnSlotValue(KB.s_title);
			if(pnam.equals( args[1] )){
				DrlEmitter de = new DrlEmitter(pk);
				de.emitRules( args[2] );
				System.out.println(args[1]+" emmited to "+args[2]);
				return;
			}
		}
		System.out.println("Instance not found "+args[1]);
	}

}
