/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * Copyright (C) 2007 Ruslan P. Sorokin (sorokin@oogis.ru)
 * and Object-Oriented Geo-Information System Research Laboratory (http://www.oogis.ru)
 * All Rights Reserved.
 */
package ru.oogis.dro;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.Reader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;

import edu.stanford.smi.protege.model.Instance;
import edu.stanford.smi.protege.model.KnowledgeBase;
import edu.stanford.smi.protege.model.Project;

public class DrlParser {
	
	public final static String DIALECT = "dialect";
	public final static String DATE_EFFECTIVE = "date-effective";
	public final static String DATE_EXPIRES = "date-expires";
	public final static String ACTIVATION_GROUP = "activation-group";
	public final static String AGENDA_GROUP = "agenda-group";
	public final static String AUTO_FOCUS = "auto-focus";
	public final static String DURATION = "duration";
	public final static String END = "end";
	public final static String EXPANDER = "expander";
	public final static String FUNCTION = "function";
	public final static String GLOBAL = "global";
	public final static String IMPORT = "import";
	public final static String NO_LOOP = "no-loop";
	public final static String PACKAGE = "package";
	public final static String RULE = "rule";
	public final static String SALIENCE = "salience";
	public final static String THEN = "then";
	public final static String WHEN = "when";
	
	Instance packInst;
	KnowledgeBase kb;
	Collection<String> errors;
	
	public DrlParser(KnowledgeBase kb) {
		this.kb = kb;
	}
	
	public DrlParser(KnowledgeBase kb, Instance packInst) {
		this.kb = kb;
		this.packInst = packInst;
	}
	
	public Instance getPackInst() {
		if(packInst == null)
			packInst = kb.createInstance(null, KB.c_Package);
		return packInst;
	}
	@SuppressWarnings("unchecked")
	public void parse(Reader reader) throws Exception
	{
		String lex;
		errors = new ArrayList();
		LexReader lr = new LexReader(reader, errors);
		 while(!lr.getNextLex().equals(PACKAGE))
			 ;
		 createPackage(lr.getNextLex());
		 while((lex = lr.getNextLex()) != LexReader.EOF){
			 if(lex.equals(EXPANDER))
				 createExpander(lr.getNextLex());
			 else if(lex.equals(IMPORT))	
				 createImport(lr.getNextLex());
			 else if(lex.equals(GLOBAL))	
				 createGlobal(lr.getNextLex(), lr.getNextLex());
			 else if(lex.equals(FUNCTION))
				 createFunction(lr);
			 else if(lex.equals(RULE))
				 createRule(lr);
		 }
	}
	public void parse(String path) throws Exception
	{
		parse(new FileReader(path));
	}
	public void createRule(LexReader lr) throws IOException
	{
		String lex;
		Rule rule = new Rule();
		rule.name = lr.getNextLex();
		 while((lex = lr.getNextLex()) != LexReader.EOF){
			 if(lex.equals(SALIENCE))
				 rule.salience = lr.getNextLex();
			 else if(lex.equals(DURATION))
				 rule.duration = lr.getNextLex();
			 else if(lex.equals(NO_LOOP))
				 rule.no_loop = lr.getNextLex();
			 else if(lex.equals(AGENDA_GROUP))
				 rule.agenda_group = lr.getNextLex();
			 else if(lex.equals(AUTO_FOCUS))
				 rule.auto_focus = lr.getNextLex();
			 else if(lex.equals(DIALECT))
				 rule.dialect = lr.getNextLex();
			 else if(lex.equals(DATE_EFFECTIVE))
				 rule.date_effective = lr.getNextLex();
			 else if(lex.equals(DATE_EXPIRES))
				 rule.date_expires = lr.getNextLex();
			 else if(lex.equals(ACTIVATION_GROUP))
				 rule.activation_group = lr.getNextLex();
			 else if(lex.equals(WHEN)){
				 rule.when = lr.getLinesTill(THEN);
				 rule.then = lr.getLinesTill(END);
				 addRuleInstance(rule);
				 return;
			 }
		 }
	}
	@SuppressWarnings("unchecked")
	public void createFunction(LexReader lr) throws IOException
	{
		String retyp = lr.getNextLex();
		String fname = lr.getNextLex();
		if(lr.getNextLex().equals("(")){
			String parms = lr.getLinesTill(")");
			if(lr.getNextLex().equals("{")){
				String code = lr.getLinesTill("}");
				Instance pi = getPackInst();
				Collection<Instance> funs = new ArrayList<Instance>(pi.getOwnSlotValues(KB.s_functions));
				Instance fun = kb.createInstance(null, KB.c_Function);
				fun.setOwnSlotValue(KB.s_title, fname);
				fun.setOwnSlotValue(KB.s_return_type, retyp);
				fun.setOwnSlotValue(KB.s_params, parms);
				fun.setOwnSlotValue(KB.s_source, code);
				funs.add(fun);
				pi.setOwnSlotValues(KB.s_functions, funs);
				return;
			}
		}
		System.out.println("LexReader: Syntax error in function definition "+retyp+" "+fname);
	}
	public void createPackage(String lex) throws IOException
	{
		if(lex.endsWith(";"))
			lex = lex.substring(0, lex.length()-1);
		Instance pi = getPackInst();
		pi.setOwnSlotValue(KB.s_title
				, lex);
	}
	public void createExpander(String lex) throws IOException
	{
		Instance pi = getPackInst();
		pi.setOwnSlotValue(KB.s_expander, lex);
	}
	@SuppressWarnings("unchecked")
	public void createImport(String lex) throws IOException
	{
		Instance pi = getPackInst();
		Collection<String> imps = new ArrayList(pi.getOwnSlotValues(KB.s_imports));
		imps.add(lex);
		pi.setOwnSlotValues(KB.s_imports, imps);
	}
	@SuppressWarnings("unchecked")
	public void createGlobal(String clazz, String identifier) throws IOException
	{
		Instance pi = getPackInst();
		Collection<Instance> glbs = new ArrayList(pi.getOwnSlotValues(KB.s_globals));
		Instance glb = kb.createInstance(null, KB.c_Global);
		glb.setOwnSlotValue(KB.s_title, identifier);
		glb.setOwnSlotValue(KB.s_class, clazz);
		glbs.add(glb);
		pi.setOwnSlotValues(KB.s_globals, glbs);
	}
	@SuppressWarnings("unchecked")
	public void addRuleInstance(Rule rule)
	{
		Instance rulinst = kb.createInstance(null, KB.c_Rule);
		rulinst.setOwnSlotValue(KB.s_title, rule.name);
		rulinst.setOwnSlotValue(KB.s_agenda_group, rule.agenda_group);
		if(rule.salience != null)
			rulinst.setOwnSlotValue(KB.s_salience, new Integer(rule.salience));
		if(rule.duration != null)
			rulinst.setOwnSlotValue(KB.s_duration, new Integer(rule.duration));
		if(rule.no_loop != null)
			rulinst.setOwnSlotValue(KB.s_no_loop, new Boolean(rule.no_loop));
		if(rule.auto_focus != null)
			rulinst.setOwnSlotValue(KB.s_auto_focus, new Boolean(rule.auto_focus));
		if(rule.agenda_group != null)
			rulinst.setOwnSlotValue(KB.s_agenda_group, rule.agenda_group);
		if(rule.activation_group != null)
			rulinst.setOwnSlotValue(KB.s_activation_group, rule.activation_group);
		if(rule.date_effective != null)
			rulinst.setOwnSlotValue(KB.s_date_effective, rule.date_effective);
		if(rule.date_expires != null)
			rulinst.setOwnSlotValue(KB.s_date_expires, rule.date_expires);
		if(rule.dialect != null)
			rulinst.setOwnSlotValue(KB.s_dialect, rule.dialect);
		rulinst.setOwnSlotValue(KB.s_when, rule.when);
		rulinst.setOwnSlotValue(KB.s_then, rule.then);
		Instance pi = getPackInst();
		Collection<Instance> rules = new ArrayList(pi.getOwnSlotValues(KB.s_rules));
		rules.add(rulinst);
		pi.setOwnSlotValues(KB.s_rules, rules);
	}
	
	public Collection<String> getErrors() {
		return errors;
	}

	@SuppressWarnings("unchecked")
	public static void main(String[] args) throws Exception {
		if(args.length < 2){
			System.out.println("Usage: java -cp .. ru.oogis.dro.DrlParser <path-to-project> <path-to-drl-file>");
		}
		Collection errors = new ArrayList();
		Project prj = Project.loadProjectFromFile( args[0], errors );
		System.out.println("load errors "+errors.size());
		if(errors.size() > 0){
			for (Iterator iter = errors.iterator(); iter.hasNext();) {
				Object element = iter.next();
				System.out.println("error: "+element);
			}
			return;
		}
		DrlParser dp = new DrlParser(prj.getKnowledgeBase());
		dp.parse( args[1] );
		System.out.println("parse errors "+dp.getErrors().size());
		if(dp.getErrors().size() > 0){
			for (Iterator iter = dp.getErrors().iterator(); iter.hasNext();) {
				Object element = iter.next();
				System.out.println("error: "+element);
			}
			return;
		}
		prj.save( errors );
		System.out.println("save errors "+errors.size());
		if(errors.size() > 0){
			for (Iterator iter = errors.iterator(); iter.hasNext();) {
				Object element = iter.next();
				System.out.println("error: "+element);
			}
			return;
		}
	}
	class Rule {
		String name,
			dialect,
			salience,
			no_loop,
			agenda_group,
			auto_focus,
			duration,
			date_effective,
			date_expires,
			activation_group,
			when,
			then;
		
	}
	class LexReader extends BufferedReader{
		final static String EOF = "EOF";
		String line = "";
		Collection<String> errors;

		public LexReader(Reader in, Collection<String> errors) {
			super(in);
			this.errors = errors;
		}
		public String getNextLine() throws IOException{
			if(ready()){
				line = readLine();
				return line;
			}else{
				errors.add("LexReader: EOF unexpectedly while giving next line!");
				return null;
			}
		}
		public String getLinesTill(String word) throws IOException{
			StringBuffer sb = new StringBuffer();
			while(true){
				int ind = line.indexOf(word);
				if(ind >= 0){
					sb.append(line.substring(0, ind)+'\n');
					line = line.substring(ind+word.length());
					return sb.toString();
				}
				sb.append(line+'\n');
				do{
					if(ready())
						line = readLine();
					else{
						errors.add("LexReader: EOF unexpectedly reached while searching "+word);
						return null;
					}
				}while(line == null || line.startsWith("#"));
			}
		}
		public String getNextLex() throws IOException{
			while(true){
				line = line.trim();
				if(line.length() > 0){
					char ch0 = line.charAt(0);
					// leftpar
					if(ch0 == '(' || ch0 == ')' || ch0 == '{' || ch0 == '}' ){
						String retval =  line.substring(0,1);
						line = line.substring(1, line.length());
						return retval;
					// lexeme = string in double quotes, that will be striped
					} else if(ch0 == '\"'){
						int dubquo2ind = line.indexOf('\"', 1);
						if(dubquo2ind > 0){
							String retval = line.substring(1, dubquo2ind);
							line = line.substring(dubquo2ind+1);
							return retval;
						} else {
							errors.add("LexReader: second double quote not found in line: "+line);
							String retval = line;
							line = "";
							return retval;
						}
					// lexeme = symbol
					} else {
						// search for dividers
						int[] inds = new int[6];
						inds[0] = line.indexOf(' ', 1);
						inds[1] = line.indexOf('\t', 1);
						inds[2] = line.indexOf('(', 1);
						inds[3] = line.indexOf(')', 1);
						inds[4] = line.indexOf('{', 1);
						inds[5] = line.indexOf('}', 1);
						Arrays.sort(inds);
						for (int i = 0; i < inds.length; i++) {
							if(inds[i] > 0){
								String retval = line.substring(0, inds[i]);
								if(line.charAt(inds[i]) == ' ' || line.charAt(inds[i]) == '\t')
										inds[i] += 1;
								line = line.substring(inds[i]);
								return retval;
							}
						}
						String retval = line;
						line = "";
						return retval;
					}
				}
				do{
					if(ready())
						line = readLine();
					else
						return EOF;
				}while(line == null || line.startsWith("#"));
			}
		}
	}
}
