/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * Copyright (C) 2004 Sorokin R.P. (sorokinru@iias.spb.su)
 * and Object-oriented Geoinformatics Systems Laboratory (http://niggis.iias.spb.su)
 * All Rights Reserved.
 */

package ru.igis.omtab.ra;

import ru.igis.omtab.*;
import com.bbn.openmap.omGraphics.*;
import edu.stanford.smi.protege.model.*;
/**
 * Reference realisation of Repeat Action interface
 * Simulate movement of spacial objects under influence of Field
 * See Field class
 * @author ru
 */
public class OMTPolyOnFieldRepeatAction extends ProtegeRepeatAction {
    private OMTPoly poly;
    private Field field;

    /**
     * Creates a new instance of PolyOnField RepeatAction
     * @param p OMTPoly object
     * @param f Field object
     */
    public OMTPolyOnFieldRepeatAction(OMTPoly p, Field f) {
        poly = p;
        field = f;
    }

    /** Creates a new instance of OMTPolyOnFieldRepeatAction */
    public OMTPolyOnFieldRepeatAction() {}

    /**
     * Method implementing periodically repeating action
     * Here is moving points along the Field of vectors by one step
     * @param time time elapsed between steps
     */
    public void repeat(long time) {
        if(poly!=null)
            synchronized (poly) {
                OMPoly lm = (OMPoly)poly.getLocationMarker();
                double[] llp = lm.getLatLonArray();
                llp = field.moveLLPArray(llp,time);
                lm.setLocation(llp,OMGraphicConstants.RADIANS);
            }
    }

    /**
     * Method filling information to this RepeatAction object
     * from Protege Instance
     * @param instance Protege Instance describing this Repeat Action
     * @param kb Protege Knowledge Base
     * @param runa RuNA Object
     * @throws Exception -
     */
    public void fillFromProtege(Instance instance, KnowledgeBase kb, RuNA runa) throws Exception{
        name = (String)instance.getOwnSlotValue(kb.getSlot(Ontology.S_LABEL));
        interval = (String)instance.getOwnSlotValue(kb.getSlot(Ontology.S_INTERVAL));
        Instance moinst = (Instance)instance.getOwnSlotValue(kb.getSlot(Ontology.S_POLY));
        poly = (OMTPoly)Util.findOrAdd(moinst,runa);
        moinst = (Instance)instance.getOwnSlotValue(kb.getSlot(Ontology.S_FIELD));
        field = (Field)Util.findOrAdd(moinst,runa);
    }

    /**
     * Method called while removing this Repeat Action object
     * Should clear linked to it objects
     * @param runa RuNA object
     */
    public void removeObjects(RuNA runa){
        runa.removeMapOb(poly,false);
        runa.removeMapOb(field,false);
    }
}
